from unittest.mock import Mock, patch

import pytest
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.by import By

from src.linkedin_handler import LinkedInConnectorClass


class TestLinkedInConnectorUnit:
    @pytest.fixture(scope="class")
    def mock_connector(self):
        with patch('selenium.webdriver.Chrome') as MockWebDriver:
            mock_driver = Mock()

            # Mock WebDriver
            mock_driver.maximize_window = Mock()
            mock_driver.get = Mock()
            mock_driver.quit = Mock()
            mock_driver.find_element = Mock()

            mock_elements = {
                'username': Mock(),
                'password': Mock(),
                "//button[@type='submit']": Mock(),
                "//button[contains(text(), 'Connect')]": Mock(),
                "//button[@aria-label='Add a note']": Mock(),
                "//button[@aria-label='Send now']": Mock(),
                'custom-message': Mock()
            }

            # Add send_keys method to the mock elements for text entry
            for element in mock_elements.values():
                element.send_keys = Mock()

            # Mock find_element
            def find_element_effect(by, locator):
                if locator in mock_elements:
                    return mock_elements[locator]
                else:
                    raise NoSuchElementException
                
            mock_driver.find_element.side_effect = find_element_effect

            # Mock WebDriverWait
            mock_wait = Mock()
            def mock_until(condition):
                if 'presence_of_element_located' in str(condition):
                    # Return a single mock element for presence_of_element_located
                    return next(iter(mock_elements.values()))
                else:
                    # Return a list of mock elements for other conditions
                    return list(mock_elements.values())
            mock_wait.until.side_effect = mock_until



            mock_driver.wait = mock_wait

            MockWebDriver.return_value = mock_driver

            with patch('src.linkedin_handler.LinkedInConnectorClass.__init__', return_value=None):
                connector = LinkedInConnectorClass("path/to/chromedriver")
                connector.driver = mock_driver
                connector.wait = mock_wait
                yield connector

    @pytest.mark.parametrize("username, password", [
        ("test_username", "test_password"),
        ("another_user", "another_pass")
    ])
    def test_login(self, mock_connector, username, password):
        mock_connector.login(username, password)

        # Assert that the necessary elements are interacted with
        mock_connector.driver.find_element.assert_called_with(By.ID, 'username')
        mock_connector.driver.find_element.assert_called_with(By.ID, 'password')
        mock_connector.driver.find_element.assert_called_with(By.XPATH, "//button[@type='submit']")

    @pytest.mark.parametrize("profile_url, note", [
        ("https://www.linkedin.com/in/example1", "Hi, this is a test connection request."),
        ("https://www.linkedin.com/in/example-name-1234", "Hello, let's connect on LinkedIn.")
    ])
    def test_send_connection_request(self, mock_connector, profile_url, note):
        mock_connector.send_connection_request(profile_url, note)

        # Assert that the profile page is navigated to
        mock_connector.driver.get.assert_called_with(profile_url)

        # Assert interactions with elements for sending a connection request
        mock_connector.driver.find_element.assert_any_call(By.XPATH, "//button[contains(text(), 'Connect')]")
        mock_connector.driver.find_element.assert_any_call(By.XPATH, "//button[@aria-label='Add a note']")
        mock_connector.driver.find_element.assert_any_call(By.ID, 'custom-message')
        mock_connector.driver.find_element.assert_any_call(By.XPATH, "//button[@aria-label='Send now']")




from unittest.mock import ANY, MagicMock, patch

import pytest

from src.email_handler import \
    EmailHandler  # Adjusted to import from your module


@pytest.fixture
def email_handler():
    return EmailHandler("test@example.com", "password")

def test_initialization(email_handler):
    assert email_handler.sender_email == "test@example.com"
    assert email_handler.sender_password == "password"

@patch("src.email_handler.smtplib.SMTP")
def test_send_email_success(mock_smtp, email_handler):
    mock_server = MagicMock()
    mock_smtp.return_value.__enter__.return_value = mock_server

    email_handler.send("Test content", "recipient@example.com", "Test Subject")
    
    mock_server.starttls.assert_called_once()
    mock_server.login.assert_called_once_with("test@example.com", "password")
    mock_server.sendmail.assert_called_once()
    # Add any other assertions you deem necessary


@patch("src.email_handler.smtplib.SMTP")
@patch("src.email_handler.logger")
def test_send_email_exception(mock_logger, mock_smtp, email_handler):
    mock_server = MagicMock()
    mock_server.sendmail.side_effect = Exception("Test exception")
    mock_smtp.return_value.__enter__.return_value = mock_server

    email_handler.send("Test content", "recipient@example.com", "Test Subject")

    mock_logger.exception.assert_called_once_with(ANY)


import logging
import os

import docx
import pandas as pd
from dotenv import find_dotenv, load_dotenv
from PyPDF2 import PdfReader

# Setting up logger
logger = logging.getLogger(__name__)

TEMPLATES = ["cover_letter_template", "resume_template", "email_template", "linkedin_note_template"]

def get_file_content(path: str) -> str:
    """Get the content of a file."""
    try:
        # Ensure the file is .txt, .docx, or .pdf
        if not path.endswith((".txt", ".docx", ".pdf")):
            raise ValueError("Only .txt, .docx, and .pdf files are supported.")

        # Read the file content based on the file type
        if path.endswith(".txt"):
            with open(path, "r", encoding="utf-8") as file:
                content = file.read()
        elif path.endswith(".pdf"):
            with open(path, "rb") as file:
                pdf_reader = PdfReader(file)
                content = "\n".join([pdf_reader.pages[page].extract_text() for page in range(len(pdf_reader.pages))])
        elif path.endswith(".docx"):
            doc = docx.Document(path)
            content = "\n".join([paragraph.text for paragraph in doc.paragraphs])
        else:
            raise ValueError(f"Invalid file type: {path}")
        return content
    except Exception as e:
        logger.exception(f"Error reading file: {e}")
        return ""


def generate_function_description(name: str, description: str, *args: tuple[str, str]) -> list[dict]:
    """
    This function generates a function description for the OpenAPI specification.

    Args:
    - name (str): Name of the function.
    - description (str): Description of the function.
    - *args (tuple): Tuple of tuples containing the function arguments and their descriptions.

    Returns:
    - list[dict]: List of function descriptions in the OpenAPI specification format.
    """
    properties = {}
    required = []
    for arg_name, arg_description in args:
        properties[arg_name] = {"type": "string", "description": arg_description}
        required.append(arg_name)
    function_description = [
        {
            "name": name,
            "description": description,
            "parameters": {"type": "object", "properties": properties},
            "required": required,
        }
    ]

    return function_description


def load_prompt(prompt_name: str, prompt_args: dict) -> str:
    """
    Load prompt for a given prompt name.

    Args:
    - prompt_name (str): Name of the prompt txt file
    - prompt_args (dict): Dictionary of arguments to be passed to the prompt.

    Returns:
    - str: Prompt content in an f-string format.
    """
    if prompt_name == "generate_custom_contents":
        # Load the content from the file "generate_custom_contents.txt" and convert it to an f-string
        with open("prompts/generate_custom_contents.txt", "r") as f:
            prompt = f.read()
    else:
        raise ValueError(f"Invalid prompt name: {prompt_name}")

    return prompt.format(**prompt_args)


def create_rich_text_dict(content: str) -> dict:
    """
    Create a rich text dictionary for Notion API.

    Args:
    - content (str): Content of the rich text.

    Returns:
    - dict: Rich text dictionary.
    """
    return {
        "type": "text",
        "text": {"content": content},
    }


def create_job_folder(job:pd.Series, destination:str = "job_applications"):
    """
    Create a folder for the job application process.
    Add relevant files to the folder.
    """
    # Create a folder for the job application
    job_folder = f"{destination}/{job['Company Name']}_{job['Position']}"
    os.makedirs(job_folder, exist_ok=True)

    # Save the template files to the folder as docx files
    # TODO: Add logic to generate updated templates and save them to the folder
    templates = {
        "cover_letter": job["Cover Letter Template"],
        "resume": job["Resume Template"],
        "email": job["Email Template"],
        "linkedin_note": job["LinkedIn Note Template"],
    }
    for template_name, template_content in templates.items():
        with open(f"{job_folder}/{template_name}.docx", "w") as f:
            f.write(template_content)




def get_argument_value(arg_name, args):
    """
    Check if the argument is provided via CLI or as an environment variable.
    Raise an error if the argument is not found.
    """
    arg_value = getattr(args, arg_name, None) or os.getenv(arg_name)
    if arg_value is None:
        raise ValueError(f"Required argument '{arg_name}' not provided.")
    return arg_value

def validate_arguments(args) -> dict:
    """
    Validate the arguments passed to the CLI. 
    Check if all arguments are present and valid either from the CLI or as an environment variable.
    Create a dictionary of the arguments and their updated values.
    """
    load_dotenv(find_dotenv())
    validate_args = {}

    required_args = {
        # OpenAI arguments
        "OPENAI_API_KEY": "Openai api key",
        "OPENAI_MODEL": "Openai model to use",
        "OPENAI_URL": "Openai url",

        # Google Sheets arguments
        "GOOGLE_API_CREDENTIALS_FILE": "Path to the credentials file for google api",
        "GOOGLE_SHEET_NAME": "Name of the google sheet to read jobs from",

        # Document arguments
        "RESUME_PATH": "Path to resume",
        "COVER_LETTER_PATH": "Path to cover letter",
        
    }

    gmail_args = {
        "GMAIL_ADDRESS": "Gmail address to send emails from",
        "GMAIL_PASSWORD": "Password to gmail account",
        "EMAIL_CONTENT": "Email content",
    }

    linkedin_args = {
        "CHROMEDRIVER_PATH": "Path to the selenium driver",
        "LINKEDIN_USERNAME": "LinkedIn username",
        "LINKEDIN_PASSWORD": "LinkedIn password",
        "INTERACTIVE": "Run in interactive mode/show browser"
    }

    # Check if all required arguments are provided
    for arg_name, arg_description in required_args.items():
        if arg_name not in args.__dict__ and os.getenv(arg_name) is None:
            raise ValueError(f"Required argument '{arg_name}' not provided.")
        validate_args[arg_name] = get_argument_value(arg_name, args)
        
    # Check if all Gmail arguments are provided
    if args.USE_GMAIL:
        for arg_name, arg_description in gmail_args.items():
            if arg_name not in args.__dict__ and os.getenv(arg_name) is None:
                raise ValueError(f"Gmail argument '{arg_name}' not provided.")
            validate_args[arg_name] = get_argument_value(arg_name, args)
            
    # Check if all LinkedIn arguments are provided
    if args.USE_LINKEDIN:
        for arg_name, arg_description in linkedin_args.items():
            if arg_name not in args.__dict__ and os.getenv(arg_name) is None:
                raise ValueError(f"LinkedIn argument '{arg_name}' not provided.")
            validate_args[arg_name] = get_argument_value(arg_name, args)

    return validate_args



import logging
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

# Setting up logger
logger = logging.getLogger(__name__)

class EmailHandler:
    def __init__(self, sender_email: str, sender_password: str):
        """
        Initialize EmailHandler object with Email credentials.
        """
        self.sender_email = sender_email
        self.sender_password = sender_password

    def send(
        self,
        content: str,
        recepient_email: str,
        subject: str = "Request for Job Networking Opportunity",
    ):
        """
        Send an email with the given content and subject.

        Args:
        - content (str): Message Content.
        - email (str): Recipient email address.
        - subject (str): Message subject line.
        """
        message = MIMEMultipart()
        message["From"] = self.sender_email
        message["To"] = recepient_email
        message["Subject"] = subject
        message.attach(MIMEText(content, "plain"))

        try:
            with smtplib.SMTP("smtp.gmail.com", 587) as server:
                server.starttls()
                server.login(self.sender_email, self.sender_password)
                server.sendmail(self.sender_email, recepient_email, message.as_string())
                logger.info(f"Message sent to {recepient_email}")
        except Exception as e:
            logger.exception(f"Error occurred while sending email: {e}")


import logging
import os
from typing import Optional

import pandas as pd
from pytest import param
from tqdm import tqdm

from src.google_sheets_handler import GoogleSheetsHandler
from src.utils import create_job_folder

tqdm.pandas()
logger = logging.getLogger(__name__)
class JobProcessor:
    """
    JobProcessor class to process job applications.
    """
    def __init__(
        self,
        **kwargs,
    ):
        """
        Initialize JobProcessor class.
        """

        self.jobs_df = pd.DataFrame()
        for key, value in kwargs.items():
            setattr(self, key, value)
        logger.info("JobProcessor initialized.")
        self.gc = GoogleSheetsHandler(self.GOOGLE_API_CREDENTIALS_FILE)
        logger.info("Google Sheets Sevice Account connected.")
        

    def process_jobs(self):
        """
        Main function to process jobs based on their status.
        """
        try:
            logger.info("Processing jobs...")
            self.jobs_df = self.get_all_jobs()

            logger.info("Finding jobs with missing contacts...")
            self.find_jobs_with_missing_contacts()

            logger.info("Generating custom contents for jobs...")
            self.generate_content_for_jobs()

            logger.info("Updating Google Sheet to reflect content generation status...")
            self.gc.update_gsheet_from_dataframe(self.jobs_df)

            logger.info("Processing Content Generated jobs...")
            self.process_content_generated_jobs()

            logger.info("Updating Google Sheet to reflect email and LinkedIn connection status...")
            self.gc.update_gsheet_from_dataframe(self.jobs_df)
            
            logger.info("Job processing complete.")

        except Exception as e:
            logger.error(f"Error while processing jobs: {str(e)}")
            raise e

    def get_all_jobs(self) -> pd.DataFrame:
        """
        Get all jobs from the Google Sheet.
        """
        gsheet = self.gc.get_gsheet(self.GOOGLE_SHEET_NAME)
        jobs = pd.DataFrame(gsheet.sheet1.get_all_records())
        logger.info(f"Found {len(jobs)} jobs in Google Sheet.")

        return jobs
    
    def _get_jobs_with_status(self, status: str) -> pd.DataFrame:
        """
        Get jobs with a specific status.
        """
        jobs_df = self.jobs_df[self.jobs_df['Status'] == status].copy()
        if jobs_df.empty:
            logger.info(f"No jobs with '{status}' status found.")
        else:
            logger.info(f"Found {len(jobs_df)} jobs with '{status}' status.")
        return jobs_df

    def find_jobs_with_missing_contacts(self):
        """
        Process jobs with "New Job" and "Contact Required" status.
        For all new jobs, if email or LinkedIn contact is not provided, set status to "Contact Required"
        For all new jobs, if email or LinkedIn contact is provided, set status to "Contact Ready"
        """
        new_jobs_df = self._get_jobs_with_status('New Job')
        contact_required_jobs_df = self._get_jobs_with_status('Contact Required')

        jobs_df = pd.concat([new_jobs_df, contact_required_jobs_df])
        
        has_email = jobs_df['Email'].str.strip() != ""
        has_linkedin_contact = jobs_df['LinkedIn Contact'].str.strip() != ""
        jobs_df.loc[~(has_email | has_linkedin_contact), 'Status'] = 'Contact Required'

        self.jobs_df.update(jobs_df)

    def generate_content_for_jobs(self):
        """
        For all jobs with custom content not generated, generate custom content and set status to "Content Generated"
        Wrapper: generate_custom_contents_wrapper
            Parameters:
            - job (Series): Pandas Series containing the details of a job.
            - openai_handler (OpenAIConnectorClass): Instance of OpenAIConnectorClass.
        """

        # Fetch all jobs with either Message Content, Message Subject, LinkedIn Note, Resume, or Cover Letter missing
        jobs_to_generate_content = self.jobs_df[
            (self.jobs_df['Status'] == 'Contact Ready') & 
            (
                (self.jobs_df['Message Content'].str.strip() == "") |
                (self.jobs_df['Message Subject'].str.strip() == "") |
                (self.jobs_df['LinkedIn Note'].str.strip() == "") |
                (self.jobs_df['Resume'].str.strip() == "") |
                (self.jobs_df['Cover Letter'].str.strip() == "")
            )
        ].copy()

        if jobs_to_generate_content.empty:
            logger.info("No jobs with missing custom content found.")
            return
        
        logger.info(f"Found {len(jobs_to_generate_content)} jobs with missing custom content.")
        # Generate custom content for each job
        from src.openai_handler import OpenAIConnectorClass
        params = {
            'openapi_key': self.OPENAI_API_KEY,
            'openapi_url': self.OPENAI_URL,
            'openapi_model': self.OPENAI_MODEL,
            'resume_path': self.RESUME_PATH,
            'cover_letter_path': self.COVER_LETTER_PATH
        }
        if self.USE_GMAIL:
            params['email_template'] = self.EMAIL_CONTENT
        if self.USE_LINKEDIN:
            params['linkedin_note_template'] = self.LINKEDIN_NOTE_TEMPLATE
        openai_handler = OpenAIConnectorClass(**params)

        def generate_custom_contents_wrapper(job: pd.Series, openai_handler: OpenAIConnectorClass) -> pd.Series:
            try:
                generated_contents = openai_handler.generate_custom_content(job)
                for key, value in generated_contents.items():
                    job[key] = value
                job['Status'] = 'Content Generated'  # Set status if no error occurs
                logger.info(f"Custom contents generated for job at Company Name {job['Company Name']}")

                create_job_folder(job) # type: ignore

            except Exception as e:
                logger.error(f"Failed to generate custom contents for job at Company Name {job['Company Name']}. Error: {str(e)}")
                job['Status'] = 'ERROR: Failed to generate custom contents'
            return job
        

        jobs_to_generate_content.progress_apply(lambda job: generate_custom_contents_wrapper(job, openai_handler), axis=1) # type: ignore
        self.jobs_df.update(jobs_to_generate_content)

    def process_content_generated_jobs(self):
        """
        Process jobs with "Content Generated" status.
        For all jobs with "Content Generated" status, send emails and LinkedIn messages and set status to "Email Sent" or "LinkedIn Connection Sent"
        For all jobs with "Email Sent" or "LinkedIn Connection Sent" status, update the Google Sheet.
        """
        
        content_generated_jobs_df = self._get_jobs_with_status('Content Generated')
        if self.USE_GMAIL:
            email_jobs_df = content_generated_jobs_df[content_generated_jobs_df['Email'].str.strip() != ""].copy()
            if not email_jobs_df.empty:
                self.send_emails(email_jobs_df)
                self.jobs_df.update(email_jobs_df)

        if self.USE_LINKEDIN:
            linkedin_jobs_df = content_generated_jobs_df[content_generated_jobs_df['LinkedIn Contact'].str.strip() != ""].copy()
            if not linkedin_jobs_df.empty:
                self.send_linkedin_connections(linkedin_jobs_df)
                self.jobs_df.update(linkedin_jobs_df)

    def send_emails(self, jobs_df: pd.DataFrame):
        """
        Send emails for each job in the DataFrame and update the status.
        Wrapper: send_email_wrapper
            Parameters:
            - job (Series): Pandas Series containing the details of a job.
            - email_handler (EmailHandler): Instance of EmailHandler.
        """
        from src.email_handler import EmailHandler
        email_handler = EmailHandler(self.GMAIL_ADDRESS, self.GMAIL_PASSWORD)
        logger.info("Email Connection Established.")
        logger.info(f"Sending emails to {len(jobs_df)} contacts...")
        def send_email_wrapper(job: pd.Series, email_handler: EmailHandler) -> pd.Series:
                try:
                    email_handler.send(
                        content=job['Message Content'],
                        recepient_email=job['Email'],
                        subject=job['Message Subject'],
                    )
                    job['Status'] = 'Email Sent'
                    logger.info(f"Email sent to {job['Email']} for job at Company Name {job['Company Name']}")
                except Exception as e:
                    logger.warning(f"Failed to send email for job at Company Name {job['Company Name']}. Error: {str(e)}")
                    job['Status'] = 'Failed to send email'
                return job
                
        jobs_df.progress_apply(lambda job: send_email_wrapper(job, email_handler), axis=1) # type: ignore
        self.jobs_df.update(jobs_df)

    def send_linkedin_connections(self, jobs_df: pd.DataFrame):
        """
        Send LinkedIn connection requests for each job in the DataFrame.
        Wrapper: send_linkedin_connection_with_message
            Parameters:
            - job (Series): Pandas Series containing the details of a job.
            - linkedin_handler (LinkedInConnectorClass): Instance of LinkedInConnectorClass.
        """        
        
        from src.linkedin_handler import LinkedInConnectorClass
        logger.info("Logging into LinkedIn...")
        linkedin_handler = LinkedInConnectorClass(self.CHROMEDRIVER_PATH, self.INTERACTIVE)
        linkedin_handler.login(self.LINKEDIN_USERNAME, self.LINKEDIN_PASSWORD)

        logger.info("LinkedIn Connection Established.")
        logger.info(f"Sending LinkedIn connection requests to {len(jobs_df)} contacts...")
        def send_linkedin_connection_with_message(job: pd.Series, linkedin_handler: LinkedInConnectorClass) -> pd.Series:
            try:
                linkedin_handler.send_connection_request(
                    profile_url=job['LinkedIn Contact'],
                    note=job['LinkedIn Note']
                )
                job['Status'] = 'LinkedIn Connection Sent'
                logger.info(f"LinkedIn connection request sent to {job['Name']} for job at Company Name {job['Company Name']}")
                
            except Exception as e:
                logger.warning(f"Failed to send LinkedIn connection request for job at Company Name {job['Company Name']}. Error: {str(e)}")
                job['Status'] = 'Failed to send LinkedIn connection request'

            return job

        jobs_df.progress_apply(lambda job: send_linkedin_connection_with_message(job, linkedin_handler), axis=1) # type: ignore
        self.jobs_df.update(jobs_df)

    def update_gsheet(self):
        """
        Update Google Sheet with the updated jobs DataFrame.
        """
        try:
            self.gc.update_gsheet_from_dataframe(self.jobs_df)
            logger.info("Google Sheet updated successfully.")
        except Exception as e:
            logger.error(f"Error while updating Google Sheet: {str(e)}")
            raise


from pathlib import Path
from typing import Optional

import gspread
import pandas as pd
from gspread.exceptions import SpreadsheetNotFound


class GoogleSheetsHandler:
    def __init__(self, credentials_file_path: str):
        """
        Initialize GoogleSheets object and establish connection with Google Sheets.
        Parameters
        ----------
        credentials_file_path : str
            Path to the credentials file for Google API.
        gsheet_name : str
            Name of the Google Sheet to read jobs from.
        """

        if not credentials_file_path.endswith(".json"):
            raise ValueError("Credentials file path must be a .json file.")
        
        credentials_file = Path(credentials_file_path)
        if not credentials_file.is_file():
            raise FileNotFoundError("Credentials file not found. Path provided: " + credentials_file_path)
        
        self.gc = gspread.service_account(filename=credentials_file)


    def get_gsheet(self, gsheet_name: str = "AIJobApply") -> gspread.Spreadsheet:
        """
        Get Google Sheet by name
        """
        try:
            return self.gc.open(gsheet_name)
        except SpreadsheetNotFound:
            raise ValueError(f"Google Sheet with name '{gsheet_name}' not found.")
        except Exception as e:
            raise ValueError(f"Error while getting Google Sheet: {str(e)}")
        
    def update_gsheet_from_dataframe(self, dataframe: pd.DataFrame, gsheet_name: str):
        """
        Update Google Sheet with the given dataframe
        Parameters
        ----------
        dataframe : pd.DataFrame
            Dataframe to update Google Sheet with.
        """
        try:
            gsheet = self.get_gsheet(gsheet_name)
            gsheet.sheet1.clear()
            gsheet.sheet1.update([dataframe.columns.values.tolist()] + dataframe.values.tolist())
        except Exception as e:
            raise ValueError(f"Error while updating Google Sheet: {str(e)}")
        
        


import logging
import re

from selenium import webdriver
from selenium.common.exceptions import (ElementNotInteractableException,
                                        NoSuchElementException,
                                        TimeoutException)
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait


class LinkedInConnectorClass:
    """
    A class to automate LinkedIn connection requests.

    Methods:
    - __init__: Initializes the class with driver, username, and password.
    - validate_input: Validates input parameters.
    - initialize_driver: Initializes the WebDriver.
    - login: Logs into LinkedIn.
    - send_connection_request: Sends a connection request to a specified LinkedIn profile.
    - click_connect_button: Clicks the "Connect" button on a LinkedIn profile.
    - add_note_and_send: Adds a note and sends the connection request.
    - enter_text: Enters text into a web element.
    - click: Clicks a web element identified by the given XPath.
    - close_browser: Closes the WebDriver browser.
    """

    def __init__(self, driver_path: str, interactive: bool = False):
        """Initializes the LinkedInConnectorClass with the given WebDriver path"""
        if not driver_path:
            raise ValueError("Driver path not provided.")
        self.logger = logging.getLogger(__name__)
        self.driver = self.initialize_driver(driver_path, interactive)
        self.wait = WebDriverWait(self.driver, 10)

    @staticmethod
    def initialize_driver(driver_path: str, interactive: bool = False):
        """Initializes the Selenium WebDriver with Chrome options."""
        options = webdriver.ChromeOptions()
        options.add_argument("--disable-extensions")
        options.add_argument("--disable-gpu")
        options.add_argument("--no-sandbox")
        options.add_argument("--disable-dev-shm-usage")
        if not interactive:
            options.add_argument("--headless")
        prefs = {"profile.managed_default_content_settings.images": 5,}
        options.add_experimental_option("prefs", prefs)
        service = webdriver.ChromeService(executable_path=driver_path)
        driver = webdriver.Chrome(service=service, options=options)
        driver.maximize_window()
        return driver

    def login(self, username: str, password: str):
        """Logs into LinkedIn using the provided username and password."""
        if not username or not password:
            raise ValueError("Linkedin Username or password not provided.")
        try:
            self.driver.get('https://www.linkedin.com/login')
            self.enter_text(By.ID, 'username', username)
            self.enter_text(By.ID, 'password', password)
            if not self.click("//button[@type='submit']"):
                self.logger.error("Login button click failed")
        except (NoSuchElementException, TimeoutException) as e:
            self.logger.error(f"Error while logging in: {e}")

    def send_connection_request(self, profile_url, note):
        """
        Sends a connection request to the given LinkedIn profile URL.
        Adds a note if provided.
        """
        self.click_connect_button(profile_url)
        self.add_note_and_send(note)

    def get_name_from_url(self, profile_url):
        """Extracts the name from the given LinkedIn profile URL."""
        # Extract the first name from the URL
        profile_identifier = re.search(r'/in/(.+?)/', profile_url).group(1)
        name_xpath = f"//a[contains(@href, '/in/{profile_identifier}/')]/h1[contains(@class, 'text-heading-xlarge')]"
        try:
            name_element = self.driver.find_element(By.XPATH, name_xpath)
        except NoSuchElementException:
            self.logger.error("Error while getting name from URL")
            return None
        name = name_element.text
        return name


    def click_connect_button(self, profile_url):
        """
        Navigates to the given profile URL and clicks the "Connect" button.
        If the button is within a dropdown, expands the dropdown first.
        """
        self.driver.get(profile_url)

        # Get the name from the profile URL
        name = self.get_name_from_url(profile_url)
        if not name:
            raise Exception("Error while getting name from URL")


        try:    
            xpath = (
                f"[contains(translate(@aria-label, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', 'abcdefghijklmnopqrstuvwxyz'), 'invite') "
                f"and contains(translate(@aria-label, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', 'abcdefghijklmnopqrstuvwxyz'), '{name.lower()}') "
                f"and contains(translate(@aria-label, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', 'abcdefghijklmnopqrstuvwxyz'), 'to connect') "
                f"and (@role='button' or self::button)]"
            )
            
            connect_buttons_xpath = f"//button{xpath}"
                
            if not self.click(connect_buttons_xpath):
                raise Exception("No Direct clickable connect buttons found, trying dropdowns")

        except:
            
            self.logger.error("No Direct clickable connect buttons found, trying dropdowns")
                
            try:
                more_actions_buttons_xpath = f"//button[contains(@aria-label, 'More actions')]"
                if not self.click(more_actions_buttons_xpath):
                    raise Exception("No more actions buttons found")
                
                connect_divs_xpath = f"//div{xpath}"  # Assuming 'xpath' is defined earlier
                if not self.click(connect_divs_xpath):
                    raise Exception("No dropdown connect buttons found")
                
            except Exception as e:
                self.logger.error(f"Error while clicking more actions and dropdown connect buttons: {e}")
                raise Exception("Error while clicking more actions and dropdown connect buttons")

                

    def add_note_and_send(self, note):
        """Adds a note to the connection request and sends it."""
        try:
            if not self.click("//button[@aria-label='Add a note']"):
                raise Exception("No Add a note button found")
            self.enter_text(By.ID, "custom-message", note)
            
            if not self.click("//button[@aria-label='Send now']"):
                raise Exception("No Send now button found")
            
        except (NoSuchElementException, TimeoutException, ElementNotInteractableException) as e:
            self.logger.error(f"Error while adding note or sending request: {e}")

    def enter_text(self, by, locator, text):
        """Enters text into a web element identified by the given locator."""
        element = self.wait.until(EC.presence_of_element_located((by, locator)))
        element.send_keys(text)

    def click(self, xpath: str) -> bool:
        try:
            elements = self.wait.until(EC.presence_of_all_elements_located((By.XPATH, xpath)))
            for element in elements:
                if element.is_displayed() and element.is_enabled():
                    element.click()
                    return True
        except (ElementNotInteractableException, TimeoutException, NoSuchElementException) as e:
            self.logger.error(f"Error clicking element with XPath {xpath}: {e}")
        return False

    def close_browser(self):
        """Closes the WebDriver browser."""
        self.driver.quit()



from unittest.mock import MagicMock, patch

import pytest

from src.openai_handler import OpenAIConnectorClass


@pytest.fixture
def openai_connector():
    return OpenAIConnectorClass('api_key', 'model')

@patch('src.openai_handler.openai.ChatCompletion.create')
@patch('src.openai_handler.openai.Completion.create')
def test_query_prompt(mock_completion_create, mock_chat_completion_create, openai_connector):
    # Mock response for the OpenAI API
    mock_response = MagicMock()
    mock_response.choices = [MagicMock()]
    mock_response.choices[0].message = {"content": "test response"}

    mock_chat_completion_create.return_value = mock_response
    mock_completion_create.return_value = mock_response

    # Test with function description
    response = openai_connector.query_prompt("test prompt", [{"function_name": "test_function"}])
    assert response == "test response"
    mock_chat_completion_create.assert_called_once()

    # Test without function description
    response = openai_connector.query_prompt("test prompt")
    assert response == "test response"
    mock_completion_create.assert_called_once()

def test_generate_custom_contents(openai_connector):
    # Assuming load_templates and load_prompt return specific values, mock these functions
    with patch('src.utils.load_templates', return_value={"cover_letter": "cover_letter", "resume": "resume", "message_content": "message_content", "message_subject_line": "message_subject_line", "linkedin_note": "linkedin_note"}), \
         patch('src.utils..load_prompt', return_value="test prompt"):
        response = openai_connector.generate_custom_contents({"Description": "desc", "Position": "pos", "Company Name": "company", "Link": "link", "Name": "name"})
        # Add assertions based on expected behavior

def test_generate_custom_contents_helper(openai_connector):
    response = openai_connector.generate_custom_contents_helper("cover_letter", "resume", "message_content", "updated_job_description", "message_subject_line", "linkedin_note")
    expected_response = {
        "Cover Letter": "cover_letter",
        "Resume": "resume",
        "Message Content": "message_content",
        "Description": "updated_job_description",
        "Message Subject": "message_subject_line",
        "LinkedIn Note": "linkedin_note"
    }
    assert response == expected_response



import re

import requests
from bs4 import BeautifulSoup


def extract_job_id_from_text(text):
    # Regex pattern to match a LinkedIn job ID
    job_id_pattern = re.compile(r'https://www\.linkedin\.com/jobs/(view|search)/[a-zA-Z0-9/?=&]*currentJobId=(\d{10,})|https://www\.linkedin\.com/jobs/view/(\d{10,})')

    match = job_id_pattern.search(text)
    return match.group(2) or match.group(3) if match else None

def scrape_linkedin_job(job_id):
    url = f"https://www.linkedin.com/jobs/view/?currentJobId={job_id}"

    # Send a GET request to the URL
    response = requests.get(url)

    # Check if the request was successful
    if response.status_code == 200:
        soup = BeautifulSoup(response.text, 'html.parser')

        # Extract job title, job description, and company name
        # Note: These selectors are hypothetical and need to be adjusted based on the actual page structure
        job_title = soup.find('h1', {'class': 'job-title-class'}).get_text(strip=True)
        job_description = soup.find('section', {'class': 'job-description-class'}).get_text(strip=True)
        company_name = soup.find('a', {'class': 'company-name-class'}).get_text(strip=True)

        return job_title, job_description, company_name
    else:
        print(f"Failed to fetch the page: Status code {response.status_code}")
        return None
    
if __name__ == '__main__':
    # Example text containing a LinkedIn job ID
    example_text1 = "Check out this job at vent.io: https://www.linkedin.com/jobs/view/3789877020"
    example_text2 = "Check out this job at vent.io: https://www.linkedin.com/jobs/search/?currentJobId=3789877020"
    example_text3 = "Check out this job at vent.io: https://www.linkedin.com/jobs/view/3789877020/?alternateChannel=search&refId=vvF5OAiyf5EF%2FdnahXISAQ%3D%3D&trackingId=xdeHBup3TfqOsjG%2Bzjd3FA%3D%3D&trk=d_flagship3_notifications"
    example_text4 = "Check out this job at vent.io: https://www.linkedin.com/jobs/search/?currentJobId=3789877020&f_AL=true&geoId=92000000&keywords=software%20engineer&location=Worldwide"

    # Extract the job ID from the text
    for example_text in [example_text1, example_text2, example_text3, example_text4]:
        job_id = extract_job_id_from_text(example_text)
        if job_id:
            # Scrape the job details
            job_title, job_description, company_name = scrape_linkedin_job(job_id)
            print(f"Job Title: {job_title}")
            print(f"Job Description: {job_description}")
            print(f"Company Name: {company_name}")
            print()
        else:
            print("No LinkedIn job ID found in the text.")


import json
import logging

import openai

from src.utils import (generate_function_description, get_file_content,
                       load_prompt)

logging.basicConfig(level=logging.INFO)

from typing import Optional


class OpenAIConnectorClass:
    def __init__(self, openapi_key: str, openai_url: str, openai_model: str, resume_path: str, cover_letter_path: str, email_template: Optional[str] = None, linkedin_note_template: Optional[str] = None):
        """
        Initialize Openai object and set API key.

        Parameters:
        - openapi_key (str): OpenAI API key.
        - openai_url (str): OpenAI API URL.
        - openai_model (str): OpenAI model to use.
        - resume_path (str): Path to resume.
        - cover_letter_path (str): Path to cover letter.
        - email_template (str): Email content. (optional)
        - linkedin_note_template (str): LinkedIn note. (optional)
        """

        openai.api_key = openapi_key
        openai.api_base = openai_url
        self.model = openai_model

        self.resume_template = get_file_content(resume_path)
        self.cover_letter_template = get_file_content(cover_letter_path)
        self.email_template = email_template
        self.linkedin_note_template = linkedin_note_template


    def query_prompt(self, prompt: str, function_description: list = [{}]) -> dict:
        """
        Query the OpenAI API with a given prompt.

        Args:
        - prompt (str): The prompt to be sent to the API.
        - function_description (list): List of dictionaries describing the functions to be used.

        Returns:
        - dict: Response from the API.
        """
        try:
            messages = [{"role": "user", "content": prompt}]
            if function_description:
                response = openai.ChatCompletion.create(
                    model=self.model,
                    messages=messages,
                    temperature=0,
                    functions=function_description,
                    function_call="auto",
                )
            else:
                response = openai.Completion.create(
                    model=self.model,
                    messages=messages,
                    temperature=0,
                )
            output = response.choices[0].message  # type: ignore
            if output.get("function_call"):
                function_name = output.function_call.name
                if function_name != "generate_custom_contents_helper":
                    raise ValueError(f"Function name {function_name} not found in the list of available functions.")
                function_call = eval(f"self.{function_name}")

                function_arguments = json.loads(output.function_call.arguments)
                output = function_call(**function_arguments)

                return output

            return output["content"]
        except Exception as e:
            logging.error(f"Error querying OpenAI API: {e}")
            return {}

    def generate_custom_content(self, job) -> dict:
        """
        Generate customized content for email, cover letter, and resume based on given inputs.
        - job: The pandas Series containing the job information.

        Returns:
        - dict: Customized contents for email, cover letter, resume, updated job description, message subject line and linkedin note
        """

        prompt_args = {
            "raw_job_description": job["Description"],
            "position": job["Position"],
            "company_name": job["Company Name"],
            "job_link": job["Link"],
            "name": job["Name"],
            "resume_template": self.resume_template,
            "cover_letter_template": self.cover_letter_template,
        }

        if self.email_template:
            prompt_args["email_template"] = self.email_template
        else:
            prompt_args["email_template"] = ""
        if self.linkedin_note_template:
            prompt_args["linkedin_note_template"] = self.linkedin_note_template
        else:
            prompt_args["linkedin_note_template"] = ""
        
        
        prompt = load_prompt(
            prompt_name="generate_custom_content",
            prompt_args=prompt_args,
        )

        try:
            function_description = generate_function_description(
                "generate_custom_contents_helper",
                "Generate custom contents",
                ("cover_letter", "Cover Letter Content"),
                ("resume", "Resume Content"),
                ("message_content", "Message Content"),
                ("updated_job_description", "Updated Job description"),
                ("message_subject_line", "Message Subject line"),
                ("linkedin_note", "LinkedIn Note")
            )
            response = self.query_prompt(prompt, function_description)
            # Check LinkedIn Note length and regenerate if necessary
            linkedin_note = response.get("LinkedIn Note")
            linkedin_prompt = "The LinkedIn note is too long. Please provide a shorter note. (200 characters or less)"
            while linkedin_note and len(linkedin_note) > 300:
                response = openai.Completion.create(
                    model=self.model,
                    prompt=linkedin_prompt,
                    temperature=0,
                )
                linkedin_note = response.choices[0].text.strip()

        except json.JSONDecodeError:
            logging.error("Error decoding JSON from OpenAI response.")
            return {}

    def generate_custom_contents_helper(
        self, 
        cover_letter: str, 
        resume: str, 
        message_content: str, 
        updated_job_description: str, 
        message_subject_line: str,
        linkedin_note: str
    ) -> dict:
        """Parse the response from OpenAI API to match the required output format."""
        return {
            "Cover Letter": cover_letter,
            "Resume": resume,
            "Message Content": message_content,
            "Description": updated_job_description,
            "Message Subject": message_subject_line,
            "LinkedIn Note": linkedin_note
        }



# test_linkedin_integration.py

import profile

import pytest

from src.linkedin_handler import LinkedInConnectorClass


@pytest.fixture(scope="module")
def connector():
    """
    Pytest fixture to create an instance of LinkedInConnectorClass.
    The class automatically handles login during initialization.
    """
    driver_path = "chromedriver/chromedriver"
    test_username = "sandeeppvn2@gmail.com"  # Use a test account
    test_password = ".iB6EXxA.n"             # Use a test account
    connector = LinkedInConnectorClass(driver_path, test_username, test_password)
    
    yield connector
    connector.close_browser()

@pytest.mark.parametrize("profile_url, note", [
    (
        "https://www.linkedin.com/in/sandeeppvn/",
        "Test note"
    )
])
def test_send_connection_request_success(connector, profile_url, note):
    """
    Test if the send_connection_request method successfully navigates to a profile 
    and sends a connection request.
    """
    connector.send_connection_request(profile_url, note)
    # Add assertions to verify the request was sent
    # Note: Actual verification might be complex due to the nature of the operation
